@@ -11,9 +11,6 @@ $mirrors = {
   },
   'Asia' => {
     'China' => {
-      'http://mirror.bjtu.edu.cn/CTAN/' => 1,
-      'http://mirror.lzu.edu.cn/CTAN/' => 1,
-      'http://mirror.neu.edu.cn/CTAN/' => 1,
       'http://mirrors.tuna.tsinghua.edu.cn/CTAN/' => 1,
       'http://mirrors.ustc.edu.cn/CTAN/' => 1
     },
@@ -38,9 +35,6 @@ $mirrors = {
     'Philippines' => {
       'http://mirror.pregi.net/tex-archive/' => 1
     },
-    'Singapore' => {
-      'http://download.nus.edu.sg/mirror/ctan/' => 1
-    },
     'Taiwan' => {
       'ftp://ftp.ccu.edu.tw/pub/tex/' => 1,
       'http://ftp.yzu.edu.tw/CTAN/' => 1
@@ -50,9 +44,6 @@ $mirrors = {
     }
   },
   'Australasia' => {
-    'Australia' => {
-      'http://ctan.unsw.edu.au/' => 1
-    },
     'New Zealand' => {
       'http://mirror.aut.ac.nz/CTAN/' => 1
     }
@@ -87,7 +78,7 @@ $mirrors = {
       'http://ctan.space-pro.be/tex-archive/' => 1,
       'http://ftp.fau.de/ctan/' => 1,
       'http://mirror.physik-pool.tu-berlin.de/tex-archive/' => 1,
-      'http://sunsite.informatik.rwth-aachen.de/ftp/pub/mirror/ctan/' => 1
+      'http://vesta.informatik.rwth-aachen.de/ftp/pub/mirror/ctan/' => 1
     },
     'Greece' => {
       'http://ftp.cc.uoc.gr/mirrors/CTAN/' => 1
@@ -97,6 +88,7 @@ $mirrors = {
     },
     'Netherlands' => {
       'http://ctan.cs.uu.nl/' => 1,
+      'http://ctan.triasinformatica.nl/' => 1,
       'http://ftp.snt.utwente.nl/pub/software/tex/' => 1
     },
     'Norway' => {
@@ -152,13 +144,11 @@ $mirrors = {
       'http://ctan.math.washington.edu/tex-archive/' => 1,
       'http://ctan.mirrors.hoobly.com/' => 1,
       'http://ctan.sharelatex.com/tex-archive/' => 1,
-      'http://get-software.net/' => 1,
       'http://mirror.hmc.edu/ctan/' => 1,
       'http://mirror.jmu.edu/pub/CTAN/' => 1,
       'http://mirror.unl.edu/ctan/' => 1,
       'http://mirror.utexas.edu/ctan/' => 1,
       'http://mirrors.acm.jhu.edu/ctan/' => 1,
-      'http://mirrors.concertpass.com/tex-archive/' => 1,
       'http://mirrors.ibiblio.org/pub/mirrors/CTAN/' => 1,
       'http://mirrors.rit.edu/CTAN/' => 1
     }
@@ -166,8 +156,7 @@ $mirrors = {
   'South America' => {
     'Brazil' => {
       'http://linorg.usp.br/CTAN/' => 1,
-      'http://repositorios.cpai.unb.br/ctan/' => 1,
-      'http://www.lasca.ic.unicamp.br/pub/ctan/' => 1
+      'http://repositorios.cpai.unb.br/ctan/' => 1
     },
     'Chile' => {
       'http://ftp.inf.utfsm.cl/pub/tex-archive/' => 1
